@@ -97,6 +97,23 @@ class LibvirtInspector(virt_inspector.Inspector):
 
             params = dict((p.get('name').lower(), p.get('value'))
                           for p in iface.findall('filterref/parameter'))
+
+            # Extract interface ID
+            try:
+                interfaceid = iface.find('virtualport').find(
+                    'parameters').get('interfaceid')
+            except AttributeError:
+                interfaceid = None
+
+            # Extract source bridge
+            try:
+                bridge = iface.find('source').get('bridge')
+            except AttributeError:
+                bridge = None
+
+            params['interfaceid'] = interfaceid
+            params['bridge'] = bridge
+
             dom_stats = domain.interfaceStats(name)
             yield virt_inspector.InterfaceStats(name=name,
                                                 mac=mac_address,
@@ -122,23 +139,35 @@ class LibvirtInspector(virt_inspector.Inspector):
     def inspect_disks(self, instance, duration):
         domain = self._get_domain_not_shut_off_or_raise(instance)
         for device in self._get_disk_devices(domain):
-            block_stats = domain.blockStats(device)
-            block_stats_flags = domain.blockStatsFlags(device, 0)
-            yield virt_inspector.DiskStats(
-                device=device,
-                read_requests=block_stats[0], read_bytes=block_stats[1],
-                write_requests=block_stats[2], write_bytes=block_stats[3],
-                errors=block_stats[4],
-                wr_total_times=block_stats_flags['wr_total_times'],
-                rd_total_times=block_stats_flags['rd_total_times'])
+            try:
+                block_stats = domain.blockStats(device)
+                block_stats_flags = domain.blockStatsFlags(device, 0)
+                yield virt_inspector.DiskStats(
+                    device=device,
+                    read_requests=block_stats[0], read_bytes=block_stats[1],
+                    write_requests=block_stats[2], write_bytes=block_stats[3],
+                    errors=block_stats[4],
+                    wr_total_times=block_stats_flags['wr_total_times'],
+                    rd_total_times=block_stats_flags['rd_total_times'])
+            except libvirt.libvirtError as ex:
+                # raised error even if lock is acquired while live migration,
+                # even it looks normal.
+                LOG.warning(_("Error from libvirt while checking blockStats, "
+                              "This may not be harmful, but please check : "
+                              "%(ex)s") % {'ex': ex})
+                pass
 
     @libvirt_utils.retry_on_disconnect
     def inspect_disk_info(self, instance, duration):
         domain = self._get_domain_not_shut_off_or_raise(instance)
         for device in self._get_disk_devices(domain):
             block_info = domain.blockInfo(device)
+            # if vm mount cdrom, libvirt will align by 4K bytes, capacity may
+            # be smaller than physical, avoid with this.
+            # https://libvirt.org/html/libvirt-libvirt-domain.html
+            disk_capacity = max(block_info[0], block_info[2])
             yield virt_inspector.DiskInfo(device=device,
-                                          capacity=block_info[0],
+                                          capacity=disk_capacity,
                                           allocation=block_info[1],
                                           physical=block_info[2])
 
