@@ -14,8 +14,8 @@
 import glanceclient
 from oslo_config import cfg
 
-from ceilometer.agent import plugin_base
 from ceilometer import keystone_client
+from ceilometer.polling import plugin_base
 
 SERVICE_OPTS = [
     cfg.StrOpt('glance',
@@ -23,20 +23,17 @@ SERVICE_OPTS = [
                help='Glance service type.'),
 ]
 
-cfg.CONF.register_opts(SERVICE_OPTS, group='service_types')
-cfg.CONF.import_group('service_credentials', 'ceilometer.keystone_client')
-
 
 class ImagesDiscovery(plugin_base.DiscoveryBase):
-    def __init__(self):
-        super(ImagesDiscovery, self).__init__()
-        conf = cfg.CONF.service_credentials
+    def __init__(self, conf):
+        super(ImagesDiscovery, self).__init__(conf)
+        creds = conf.service_credentials
         self.glance_client = glanceclient.Client(
             version='2',
-            session=keystone_client.get_session(),
-            region_name=conf.region_name,
-            interface=conf.interface,
-            service_type=cfg.CONF.service_types.glance)
+            session=keystone_client.get_session(conf),
+            region_name=creds.region_name,
+            interface=creds.interface,
+            service_type=conf.service_types.glance)
 
     def discover(self, manager, param=None):
         """Discover resources to monitor."""
