@@ -15,9 +15,7 @@
 """Common code for working with images
 """
 
-from __future__ import absolute_import
-
-from ceilometer.agent import plugin_base
+from ceilometer.polling import plugin_base
 from ceilometer import sample
 
 
@@ -46,21 +44,6 @@ class _Base(plugin_base.PollsterBase):
                         "virtual_size"])
 
 
-class ImagePollster(_Base):
-    def get_samples(self, manager, cache, resources):
-        for image in resources:
-            yield sample.Sample(
-                name='image',
-                type=sample.TYPE_GAUGE,
-                unit='image',
-                volume=1,
-                user_id=None,
-                project_id=image.owner,
-                resource_id=image.id,
-                resource_metadata=self.extract_image_metadata(image),
-            )
-
-
 class ImageSizePollster(_Base):
     def get_samples(self, manager, cache, resources):
         for image in resources:
