@@ -22,27 +22,12 @@ from oslo_log import log
 
 from ceilometer import keystone_client
 
-OPTS = [
-    cfg.BoolOpt('nova_http_log_debug',
-                default=False,
-                # Added in Mitaka
-                deprecated_for_removal=True,
-                help=('Allow novaclient\'s debug log output. '
-                      '(Use default_log_levels instead)')),
-]
-
 SERVICE_OPTS = [
     cfg.StrOpt('nova',
                default='compute',
                help='Nova service type.'),
 ]
 
-cfg.CONF.register_opts(OPTS)
-cfg.CONF.register_opts(SERVICE_OPTS, group='service_types')
-cfg.CONF.import_opt('http_timeout', 'ceilometer.service')
-cfg.CONF.import_opt('glance', 'ceilometer.image.discovery', 'service_types')
-cfg.CONF.import_group('service_credentials', 'ceilometer.keystone_client')
-
 LOG = log.getLogger(__name__)
 
 
@@ -62,32 +47,27 @@ def logged(func):
 class Client(object):
     """A client which gets information via python-novaclient."""
 
-    def __init__(self):
+    def __init__(self, conf):
         """Initialize a nova client object."""
-        conf = cfg.CONF.service_credentials
+        creds = conf.service_credentials
 
-        logger = None
-        if cfg.CONF.nova_http_log_debug:
-            logger = log.getLogger("novaclient-debug")
-            logger.logger.setLevel(log.DEBUG)
-        ks_session = keystone_client.get_session()
+        ks_session = keystone_client.get_session(conf)
 
         self.nova_client = nova_client.Client(
             version=api_versions.APIVersion('2.1'),
             session=ks_session,
 
             # nova adapter options
-            region_name=conf.region_name,
-            interface=conf.interface,
-            service_type=cfg.CONF.service_types.nova,
-            logger=logger)
+            region_name=creds.region_name,
+            endpoint_type=creds.interface,
+            service_type=conf.service_types.nova)
 
         self.glance_client = glanceclient.Client(
             version='2',
             session=ks_session,
-            region_name=conf.region_name,
-            interface=conf.interface,
-            service_type=cfg.CONF.service_types.glance)
+            region_name=creds.region_name,
+            interface=creds.interface,
+            service_type=conf.service_types.glance)
 
     def _with_flavor_and_image(self, instances):
         flavor_cache = {}
