@@ -198,7 +198,8 @@ class GnocchiPublisher(publisher.ConfigPublisherBase):
         # TODO(jd) allow to override Gnocchi endpoint via the host in the URL
         options = urlparse.parse_qs(parsed_url.query)
 
-        self.filter_project = options.get('filter_project', [True])[-1]
+        self.filter_project = options.get('filter_project', ['service'])[-1]
+        self.filter_domain = options.get('filter_domain', ['Default'])[-1]
 
         resources_definition_file = options.get(
             'resources_definition_file', ['gnocchi_resources.yaml'])[-1]
@@ -252,7 +253,8 @@ class GnocchiPublisher(publisher.ConfigPublisherBase):
             pkg_resources.resource_filename(__name__,
                                             "data/gnocchi_resources.yaml"))
 
-        archive_policy_default = data.get("archive_policy_default", "low")
+        archive_policy_default = data.get("archive_policy_default",
+                                          "ceilometer-low")
         resource_defs = []
         for resource in data.get('resources', []):
             try:
@@ -261,18 +263,19 @@ class GnocchiPublisher(publisher.ConfigPublisherBase):
                     archive_policy_default,
                     archive_policy_override,
                     plugin_manager))
-            except Exception as exc:
-                LOG.error("Failed to load resource due to error %s" %
-                          exc)
+            except Exception:
+                LOG.error("Failed to load resource due to error",
+                          exc_info=True)
         return resource_defs, data.get("archive_policies", [])
 
     def ensures_archives_policies(self):
         if not self._already_configured_archive_policies:
             for ap in self.archive_policies_definition:
                 try:
-                    self._gnocchi.archive_policy.get(ap["name"])
-                except gnocchi_exc.ArchivePolicyNotFound:
                     self._gnocchi.archive_policy.create(ap)
+                except gnocchi_exc.ArchivePolicyAlreadyExists:
+                    # created in the meantime by another worker
+                    pass
             self._already_configured_archive_policies = True
 
     @property
@@ -283,7 +286,8 @@ class GnocchiPublisher(publisher.ConfigPublisherBase):
             if self._gnocchi_project_id is None:
                 try:
                     project = self._ks_client.projects.find(
-                        name=self.filter_project)
+                        name=self.filter_project,
+                        domain=self.filter_domain)
                 except ka_exceptions.NotFound:
                     LOG.warning('filtered project not found in keystone,'
                                 ' ignoring the filter_project '
@@ -334,9 +338,10 @@ class GnocchiPublisher(publisher.ConfigPublisherBase):
         for resource_id, samples_of_resource in resource_grouped_samples:
             # NOTE(sileht): / is forbidden by Gnocchi
             resource_id = resource_id.replace('/', '_')
-
             for sample in samples_of_resource:
                 metric_name = sample.name
+                LOG.debug("Processing sample [%s] for resource ID [%s].",
+                          sample, resource_id)
                 rd = self.metric_map.get(metric_name)
                 if rd is None:
                     if metric_name not in self._already_logged_metric_names:
@@ -488,6 +493,25 @@ class GnocchiPublisher(publisher.ConfigPublisherBase):
             rd, operation = rd
             if operation == EVENT_DELETE:
                 self._delete_event(rd, event)
+            if operation == EVENT_CREATE:
+                self._create_event(rd, event)
+            if operation == EVENT_UPDATE:
+                self._update_event(rd, event)
+
+    def _update_event(self, rd, event):
+        resource = rd.event_attributes(event)
+        associated_resources = rd.cfg.get('event_associated_resources', {})
+
+        if associated_resources:
+            to_update = itertools.chain([resource], *[
+                self._search_resource(resource_type, query % resource['id'])
+                for resource_type, query in associated_resources.items()
+            ])
+        else:
+            to_update = [resource]
+
+        for resource in to_update:
+            self._set_update_attributes(resource)
 
     def _delete_event(self, rd, event):
         ended_at = timeutils.utcnow().isoformat()
@@ -506,17 +530,43 @@ class GnocchiPublisher(publisher.ConfigPublisherBase):
         for resource in to_end:
             self._set_ended_at(resource, ended_at)
 
+    def _create_event(self, rd, event):
+        resource = rd.event_attributes(event)
+        resource_type = resource.pop('type')
+
+        try:
+            self._create_resource(resource_type, resource)
+        except gnocchi_exc.ResourceAlreadyExists:
+            LOG.debug("Create event received on existing resource (%s), "
+                      "ignore it.", resource['id'])
+        except Exception:
+            LOG.error("Failed to create resource %s", resource,
+                      exc_info=True)
+
     def _search_resource(self, resource_type, query):
         try:
             return self._gnocchi.resource.search(
                 resource_type, json.loads(query))
         except Exception:
-            LOG.error("Fail to search resource type %{resource_type}s "
-                      "with '%{query}s'",
+            LOG.error("Fail to search resource type %(resource_type)s "
+                      "with '%(query)s'",
                       {'resource_type': resource_type, 'query': query},
                       exc_info=True)
         return []
 
+    def _set_update_attributes(self, resource):
+        try:
+            resource_id = resource.pop('id')
+            resource_type = resource.pop('type')
+
+            self._if_not_cached(resource_type, resource_id, resource)
+        except gnocchi_exc.ResourceNotFound:
+            LOG.debug("Update event received on unexisting resource (%s), "
+                      "ignore it.", resource['id'])
+        except Exception:
+            LOG.error("Fail to update the resource %s", resource,
+                      exc_info=True)
+
     def _set_ended_at(self, resource, ended_at):
         try:
             self._gnocchi.resource.update(resource['type'], resource['id'],
