@@ -18,6 +18,7 @@
 import abc
 import itertools
 import operator
+import threading
 
 from oslo_config import cfg
 from oslo_log import log
@@ -27,7 +28,7 @@ from oslo_utils import excutils
 import six
 import six.moves.urllib.parse as urlparse
 
-from ceilometer.i18n import _, _LE, _LI
+from ceilometer.i18n import _
 from ceilometer import messaging
 from ceilometer import publisher
 from ceilometer.publisher import utils
@@ -40,11 +41,13 @@ NOTIFIER_OPTS = [
                default='metering',
                help='The topic that ceilometer uses for metering '
                'notifications.',
+               deprecated_for_removal=True,
                ),
     cfg.StrOpt('event_topic',
                default='event',
                help='The topic that ceilometer uses for event '
                'notifications.',
+               deprecated_for_removal=True,
                ),
     cfg.StrOpt('telemetry_driver',
                default='messagingv2',
@@ -54,10 +57,6 @@ NOTIFIER_OPTS = [
                )
 ]
 
-cfg.CONF.register_opts(NOTIFIER_OPTS,
-                       group="publisher_notifier")
-cfg.CONF.import_opt('host', 'ceilometer.service')
-
 
 class DeliveryFailure(Exception):
     def __init__(self, message=None, cause=None):
@@ -72,9 +71,10 @@ def raise_delivery_failure(exc):
 
 
 @six.add_metaclass(abc.ABCMeta)
-class MessagingPublisher(publisher.PublisherBase):
+class MessagingPublisher(publisher.ConfigPublisherBase):
 
-    def __init__(self, parsed_url):
+    def __init__(self, conf, parsed_url):
+        super(MessagingPublisher, self).__init__(conf, parsed_url)
         options = urlparse.parse_qs(parsed_url.query)
         # the value of options is a list of url param values
         # only take care of the latest one if the option
@@ -87,10 +87,11 @@ class MessagingPublisher(publisher.PublisherBase):
             'max_queue_length', [1024])[-1])
         self.max_retry = 0
 
+        self.queue_lock = threading.Lock()
         self.local_queue = []
 
         if self.policy in ['default', 'queue', 'drop']:
-            LOG.info(_LI('Publishing policy set to %s'), self.policy)
+            LOG.info('Publishing policy set to %s', self.policy)
         else:
             LOG.warning(_('Publishing policy is unknown (%s) force to '
                           'default'), self.policy)
@@ -101,19 +102,21 @@ class MessagingPublisher(publisher.PublisherBase):
     def publish_samples(self, samples):
         """Publish samples on RPC.
 
-        :param samples: Samples from pipeline after transformation.
+        :param samples: Samples from pipeline.
 
         """
 
         meters = [
             utils.meter_message_from_counter(
-                sample, cfg.CONF.publisher.telemetry_secret)
+                sample, self.conf.publisher.telemetry_secret)
             for sample in samples
         ]
-        topic = cfg.CONF.publisher_notifier.metering_topic
-        self.local_queue.append((topic, meters))
+        topic = self.conf.publisher_notifier.metering_topic
+        with self.queue_lock:
+            self.local_queue.append((topic, meters))
 
         if self.per_meter_topic:
+            queue_per_meter_topic = []
             for meter_name, meter_list in itertools.groupby(
                     sorted(meters, key=operator.itemgetter('counter_name')),
                     operator.itemgetter('counter_name')):
@@ -121,22 +124,23 @@ class MessagingPublisher(publisher.PublisherBase):
                 topic_name = topic + '.' + meter_name
                 LOG.debug('Publishing %(m)d samples on %(n)s',
                           {'m': len(meter_list), 'n': topic_name})
-                self.local_queue.append((topic_name, meter_list))
+                queue_per_meter_topic.append((topic_name, meter_list))
+            with self.queue_lock:
+                self.local_queue.extend(queue_per_meter_topic)
 
         self.flush()
 
     def flush(self):
-        # NOTE(sileht):
-        # this is why the self.local_queue is emptied before processing the
-        # queue and the remaining messages in the queue are added to
-        # self.local_queue after in case of another call having already added
-        # something in the self.local_queue
-        queue = self.local_queue
-        self.local_queue = []
-        self.local_queue = (self._process_queue(queue, self.policy) +
-                            self.local_queue)
-        if self.policy == 'queue':
-            self._check_queue_length()
+        with self.queue_lock:
+            queue = self.local_queue
+            self.local_queue = []
+
+        queue = self._process_queue(queue, self.policy)
+
+        with self.queue_lock:
+            self.local_queue = (queue + self.local_queue)
+            if self.policy == 'queue':
+                self._check_queue_length()
 
     def _check_queue_length(self):
         queue_length = len(self.local_queue)
@@ -164,8 +168,8 @@ class MessagingPublisher(publisher.PublisherBase):
                     return []
                 current_retry += 1
                 if current_retry >= self.max_retry:
-                    LOG.exception(_LE("Failed to retry to send sample data "
-                                      "with max_retry times"))
+                    LOG.exception("Failed to retry to send sample data "
+                                  "with max_retry times")
                     raise
             else:
                 queue.pop(0)
@@ -174,13 +178,14 @@ class MessagingPublisher(publisher.PublisherBase):
     def publish_events(self, events):
         """Send an event message for publishing
 
-        :param events: events from pipeline after transformation
+        :param events: events from pipeline.
         """
         ev_list = [utils.message_from_event(
-            event, cfg.CONF.publisher.telemetry_secret) for event in events]
+            event, self.conf.publisher.telemetry_secret) for event in events]
 
-        topic = cfg.CONF.publisher_notifier.event_topic
-        self.local_queue.append((topic, ev_list))
+        topic = self.conf.publisher_notifier.event_topic
+        with self.queue_lock:
+            self.local_queue.append((topic, ev_list))
         self.flush()
 
     @abc.abstractmethod
@@ -189,11 +194,48 @@ class MessagingPublisher(publisher.PublisherBase):
 
 
 class NotifierPublisher(MessagingPublisher):
-    def __init__(self, parsed_url, default_topic):
-        super(NotifierPublisher, self).__init__(parsed_url)
+    """Publish metering data from notifier publisher.
+
+    The ip address and port number of notifier can be configured in
+    ceilometer pipeline configuration file.
+
+    User can customize the transport driver such as rabbit, kafka and
+    so on. The Notifier uses `sample` method as default method to send
+    notifications.
+
+    This publisher has transmit options such as queue, drop, and
+    retry. These options are specified using policy field of URL parameter.
+    When queue option could be selected, local queue length can be determined
+    using max_queue_length field as well. When the transfer fails with retry
+    option, try to resend the data as many times as specified in max_retry
+    field. If max_retry is not specified, by default the number of retry
+    is 100.
+
+    To enable this publisher, add the following section to the
+    /etc/ceilometer/pipeline.yaml file or simply add it to an existing
+    pipeline::
+
+        meter:
+            - name: meter_notifier
+              meters:
+                - "*"
+              sinks:
+                - notifier_sink
+        sinks:
+            - name: notifier_sink
+              publishers:
+                - notifier://[notifier_ip]:[notifier_port]?topic=[topic]&
+                  driver=driver&max_retry=100
+
+    """
+
+    def __init__(self, conf, parsed_url, default_topic):
+        super(NotifierPublisher, self).__init__(conf, parsed_url)
         options = urlparse.parse_qs(parsed_url.query)
-        topic = options.pop('topic', [default_topic])
+        topics = options.pop('topic', [default_topic])
         driver = options.pop('driver', ['rabbit'])[0]
+        self.max_retry = int(options.get('max_retry', [100])[-1])
+
         url = None
         if parsed_url.netloc != '':
             url = urlparse.urlunsplit([driver, parsed_url.netloc,
@@ -201,10 +243,10 @@ class NotifierPublisher(MessagingPublisher):
                                        urlparse.urlencode(options, True),
                                        parsed_url.fragment])
         self.notifier = oslo_messaging.Notifier(
-            messaging.get_transport(url),
-            driver=cfg.CONF.publisher_notifier.telemetry_driver,
-            publisher_id='telemetry.publisher.%s' % cfg.CONF.host,
-            topics=topic,
+            messaging.get_transport(self.conf, url),
+            driver=self.conf.publisher_notifier.telemetry_driver,
+            publisher_id='telemetry.publisher.%s' % self.conf.host,
+            topics=topics,
             retry=self.retry
         )
 
@@ -217,12 +259,12 @@ class NotifierPublisher(MessagingPublisher):
 
 
 class SampleNotifierPublisher(NotifierPublisher):
-    def __init__(self, parsed_url):
+    def __init__(self, conf, parsed_url):
         super(SampleNotifierPublisher, self).__init__(
-            parsed_url, cfg.CONF.publisher_notifier.metering_topic)
+            conf, parsed_url, conf.publisher_notifier.metering_topic)
 
 
 class EventNotifierPublisher(NotifierPublisher):
-    def __init__(self, parsed_url):
+    def __init__(self, conf, parsed_url):
         super(EventNotifierPublisher, self).__init__(
-            parsed_url, cfg.CONF.publisher_notifier.event_topic)
+            conf, parsed_url, conf.publisher_notifier.event_topic)
