@@ -14,15 +14,14 @@
 # under the License.
 
 import datetime
+from unittest import mock
 
 import jsonpath_rw_ext
-import mock
-from oslo_config import fixture as fixture_config
 import six
 
 from ceilometer import declarative
 from ceilometer.event import converter
-from ceilometer.event.storage import models
+from ceilometer.event import models
 from ceilometer import service as ceilometer_service
 from ceilometer.tests import base
 
@@ -31,10 +30,9 @@ class ConverterBase(base.BaseTestCase):
     @staticmethod
     def _create_test_notification(event_type, message_id, **kw):
         return dict(event_type=event_type,
-                    message_id=message_id,
-                    priority="INFO",
+                    metadata=dict(message_id=message_id,
+                                  timestamp="2013-08-08 21:06:37.803826"),
                     publisher_id="compute.host-1-2-3",
-                    timestamp="2013-08-08 21:06:37.803826",
                     payload=kw,
                     )
 
@@ -53,9 +51,8 @@ class ConverterBase(base.BaseTestCase):
 
     def assertHasTrait(self, event, name, value=None, dtype=None):
         traits = [trait for trait in event.traits if trait.name == name]
-        self.assertTrue(
-            len(traits) > 0,
-            "Trait %s not found in event %s" % (name, event))
+        self.assertGreater(len(traits), 0,
+                           "Trait %s not found in event %s" % (name, event))
         trait = traits[0]
         if value is not None:
             self.assertEqual(value, trait.value)
@@ -165,7 +162,7 @@ class TestTraitDefinition(ConverterBase):
         tdef = converter.TraitDefinition('test_trait', cfg,
                                          self.fake_plugin_mgr)
         t = tdef.to_trait(self.n1)
-        self.assertIs(None, t)
+        self.assertIsNone(t)
         self.nothing_plugin_class.assert_called_once_with()
         self.nothing_plugin.trait_values.assert_called_once_with([
             ('payload.instance_id', 'id-for-instance-0001'),
@@ -264,21 +261,21 @@ class TestTraitDefinition(ConverterBase):
         tdef = converter.TraitDefinition('test_trait', cfg,
                                          self.fake_plugin_mgr)
         t = tdef.to_trait(self.n1)
-        self.assertIs(None, t)
+        self.assertIsNone(t)
 
     def test_to_trait_null(self):
         cfg = dict(type='text', fields='payload.instance_id2')
         tdef = converter.TraitDefinition('test_trait', cfg,
                                          self.fake_plugin_mgr)
         t = tdef.to_trait(self.n1)
-        self.assertIs(None, t)
+        self.assertIsNone(t)
 
     def test_to_trait_empty_nontext(self):
         cfg = dict(type='datetime', fields='payload.bogus_date')
         tdef = converter.TraitDefinition('test_trait', cfg,
                                          self.fake_plugin_mgr)
         t = tdef.to_trait(self.n1)
-        self.assertIs(None, t)
+        self.assertIsNone(t)
 
     def test_to_trait_multiple_null_missing(self):
         cfg = dict(type='text', fields=['payload.not_here_boss',
@@ -286,7 +283,7 @@ class TestTraitDefinition(ConverterBase):
         tdef = converter.TraitDefinition('test_trait', cfg,
                                          self.fake_plugin_mgr)
         t = tdef.to_trait(self.n1)
-        self.assertIs(None, t)
+        self.assertIsNone(t)
 
     def test_missing_fields_config(self):
         self.assertRaises(declarative.DefinitionException,
@@ -400,9 +397,9 @@ class TestEventDefinition(ConverterBase):
     def test_to_event(self):
         dtype = models.Trait.TEXT_TYPE
         cfg = dict(event_type='test.thing', traits=self.traits_cfg)
-        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr)
+        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr, [])
 
-        e = edef.to_event(self.test_notification1)
+        e = edef.to_event('INFO', self.test_notification1)
         self.assertEqual('test.thing', e.event_type)
         self.assertEqual(datetime.datetime(2013, 8, 8, 21, 6, 37, 803826),
                          e.generated)
@@ -416,9 +413,9 @@ class TestEventDefinition(ConverterBase):
     def test_to_event_missing_trait(self):
         dtype = models.Trait.TEXT_TYPE
         cfg = dict(event_type='test.thing', traits=self.traits_cfg)
-        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr)
+        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr, [])
 
-        e = edef.to_event(self.test_notification2)
+        e = edef.to_event('INFO', self.test_notification2)
 
         self.assertHasDefaultTraits(e)
         self.assertHasTrait(e, 'instance_id',
@@ -429,9 +426,9 @@ class TestEventDefinition(ConverterBase):
     def test_to_event_null_trait(self):
         dtype = models.Trait.TEXT_TYPE
         cfg = dict(event_type='test.thing', traits=self.traits_cfg)
-        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr)
+        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr, [])
 
-        e = edef.to_event(self.test_notification3)
+        e = edef.to_event('INFO', self.test_notification3)
 
         self.assertHasDefaultTraits(e)
         self.assertHasTrait(e, 'instance_id',
@@ -444,18 +441,20 @@ class TestEventDefinition(ConverterBase):
         self.assertRaises(declarative.DefinitionException,
                           converter.EventDefinition,
                           bogus,
-                          self.fake_plugin_mgr)
+                          self.fake_plugin_mgr,
+                          [])
 
     def test_bogus_cfg_no_type(self):
         bogus = dict(traits=self.traits_cfg)
         self.assertRaises(declarative.DefinitionException,
                           converter.EventDefinition,
                           bogus,
-                          self.fake_plugin_mgr)
+                          self.fake_plugin_mgr,
+                          [])
 
     def test_included_type_string(self):
         cfg = dict(event_type='test.thing', traits=self.traits_cfg)
-        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr)
+        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr, [])
         self.assertEqual(1, len(edef._included_types))
         self.assertEqual('test.thing', edef._included_types[0])
         self.assertEqual(0, len(edef._excluded_types))
@@ -467,7 +466,7 @@ class TestEventDefinition(ConverterBase):
     def test_included_type_list(self):
         cfg = dict(event_type=['test.thing', 'other.thing'],
                    traits=self.traits_cfg)
-        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr)
+        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr, [])
         self.assertEqual(2, len(edef._included_types))
         self.assertEqual(0, len(edef._excluded_types))
         self.assertTrue(edef.included_type('test.thing'))
@@ -479,7 +478,7 @@ class TestEventDefinition(ConverterBase):
 
     def test_excluded_type_string(self):
         cfg = dict(event_type='!test.thing', traits=self.traits_cfg)
-        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr)
+        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr, [])
         self.assertEqual(1, len(edef._included_types))
         self.assertEqual('*', edef._included_types[0])
         self.assertEqual('test.thing', edef._excluded_types[0])
@@ -493,7 +492,7 @@ class TestEventDefinition(ConverterBase):
     def test_excluded_type_list(self):
         cfg = dict(event_type=['!test.thing', '!other.thing'],
                    traits=self.traits_cfg)
-        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr)
+        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr, [])
         self.assertEqual(1, len(edef._included_types))
         self.assertEqual(2, len(edef._excluded_types))
         self.assertTrue(edef.excluded_type('test.thing'))
@@ -506,7 +505,7 @@ class TestEventDefinition(ConverterBase):
     def test_mixed_type_list(self):
         cfg = dict(event_type=['*.thing', '!test.thing', '!other.thing'],
                    traits=self.traits_cfg)
-        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr)
+        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr, [])
         self.assertEqual(1, len(edef._included_types))
         self.assertEqual(2, len(edef._excluded_types))
         self.assertTrue(edef.excluded_type('test.thing'))
@@ -520,59 +519,37 @@ class TestEventDefinition(ConverterBase):
     def test_catchall(self):
         cfg = dict(event_type=['*.thing', '!test.thing', '!other.thing'],
                    traits=self.traits_cfg)
-        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr)
+        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr, [])
         self.assertFalse(edef.is_catchall)
 
         cfg = dict(event_type=['!other.thing'],
                    traits=self.traits_cfg)
-        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr)
+        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr, [])
         self.assertFalse(edef.is_catchall)
 
         cfg = dict(event_type=['other.thing'],
                    traits=self.traits_cfg)
-        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr)
+        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr, [])
         self.assertFalse(edef.is_catchall)
 
         cfg = dict(event_type=['*', '!other.thing'],
                    traits=self.traits_cfg)
-        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr)
+        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr, [])
         self.assertFalse(edef.is_catchall)
 
         cfg = dict(event_type=['*'],
                    traits=self.traits_cfg)
-        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr)
+        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr, [])
         self.assertTrue(edef.is_catchall)
 
         cfg = dict(event_type=['*', 'foo'],
                    traits=self.traits_cfg)
-        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr)
+        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr, [])
         self.assertTrue(edef.is_catchall)
 
-    @mock.patch('oslo_utils.timeutils.utcnow')
-    def test_extract_when(self, mock_utcnow):
-        now = datetime.datetime.utcnow()
-        modified = now + datetime.timedelta(minutes=1)
-        mock_utcnow.return_value = now
-
-        body = {"timestamp": str(modified)}
-        when = converter.EventDefinition._extract_when(body)
-        self.assertTimestampEqual(modified, when)
-
-        body = {"_context_timestamp": str(modified)}
-        when = converter.EventDefinition._extract_when(body)
-        self.assertTimestampEqual(modified, when)
-
-        then = now + datetime.timedelta(hours=1)
-        body = {"timestamp": str(modified), "_context_timestamp": str(then)}
-        when = converter.EventDefinition._extract_when(body)
-        self.assertTimestampEqual(modified, when)
-
-        when = converter.EventDefinition._extract_when({})
-        self.assertTimestampEqual(now, when)
-
     def test_default_traits(self):
         cfg = dict(event_type='test.thing', traits={})
-        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr)
+        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr, [])
         default_traits = converter.EventDefinition.DEFAULT_TRAITS.keys()
         traits = set(edef.traits.keys())
         for dt in default_traits:
@@ -582,7 +559,7 @@ class TestEventDefinition(ConverterBase):
 
     def test_traits(self):
         cfg = dict(event_type='test.thing', traits=self.traits_cfg)
-        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr)
+        edef = converter.EventDefinition(cfg, self.fake_plugin_mgr, [])
         default_traits = converter.EventDefinition.DEFAULT_TRAITS.keys()
         traits = set(edef.traits.keys())
         for dt in default_traits:
@@ -597,8 +574,7 @@ class TestNotificationConverter(ConverterBase):
 
     def setUp(self):
         super(TestNotificationConverter, self).setUp()
-        self.CONF = self.useFixture(fixture_config.Config()).conf
-        ceilometer_service.prepare_service(argv=[], config_files=[])
+        self.CONF = ceilometer_service.prepare_service([], [])
         self.valid_event_def1 = [{
             'event_type': 'compute.instance.create.*',
             'traits': {
@@ -628,36 +604,37 @@ class TestNotificationConverter(ConverterBase):
 
     @mock.patch('oslo_utils.timeutils.utcnow')
     def test_converter_missing_keys(self, mock_utcnow):
+        self.CONF.set_override('drop_unmatched_notifications', False,
+                               group='event')
         # test a malformed notification
         now = datetime.datetime.utcnow()
         mock_utcnow.return_value = now
         c = converter.NotificationEventsConverter(
-            [],
-            self.fake_plugin_mgr,
-            add_catchall=True)
+            self.CONF, [], self.fake_plugin_mgr)
         message = {'event_type': "foo",
-                   'message_id': "abc",
+                   'metadata': {'message_id': "abc",
+                                'timestamp': str(now)},
                    'publisher_id': "1"}
-        e = c.to_event(message)
+        e = c.to_event('INFO', message)
         self.assertIsValidEvent(e, message)
         self.assertEqual(1, len(e.traits))
         self.assertEqual("foo", e.event_type)
         self.assertEqual(now, e.generated)
 
     def test_converter_with_catchall(self):
+        self.CONF.set_override('drop_unmatched_notifications', False,
+                               group='event')
         c = converter.NotificationEventsConverter(
-            self.valid_event_def1,
-            self.fake_plugin_mgr,
-            add_catchall=True)
+            self.CONF, self.valid_event_def1, self.fake_plugin_mgr)
         self.assertEqual(2, len(c.definitions))
-        e = c.to_event(self.test_notification1)
+        e = c.to_event('INFO', self.test_notification1)
         self.assertIsValidEvent(e, self.test_notification1)
         self.assertEqual(3, len(e.traits))
         self.assertHasDefaultTraits(e)
         self.assertHasTrait(e, 'instance_id')
         self.assertHasTrait(e, 'host')
 
-        e = c.to_event(self.test_notification2)
+        e = c.to_event('INFO', self.test_notification2)
         self.assertIsValidEvent(e, self.test_notification2)
         self.assertEqual(1, len(e.traits))
         self.assertHasDefaultTraits(e)
@@ -665,117 +642,122 @@ class TestNotificationConverter(ConverterBase):
         self.assertDoesNotHaveTrait(e, 'host')
 
     def test_converter_without_catchall(self):
+        self.CONF.set_override('drop_unmatched_notifications', True,
+                               group='event')
         c = converter.NotificationEventsConverter(
-            self.valid_event_def1,
-            self.fake_plugin_mgr,
-            add_catchall=False)
+            self.CONF, self.valid_event_def1, self.fake_plugin_mgr)
         self.assertEqual(1, len(c.definitions))
-        e = c.to_event(self.test_notification1)
+        e = c.to_event('INFO', self.test_notification1)
         self.assertIsValidEvent(e, self.test_notification1)
         self.assertEqual(3, len(e.traits))
         self.assertHasDefaultTraits(e)
         self.assertHasTrait(e, 'instance_id')
         self.assertHasTrait(e, 'host')
 
-        e = c.to_event(self.test_notification2)
+        e = c.to_event('INFO', self.test_notification2)
         self.assertIsNotValidEvent(e, self.test_notification2)
 
     def test_converter_empty_cfg_with_catchall(self):
+        self.CONF.set_override('drop_unmatched_notifications', False,
+                               group='event')
         c = converter.NotificationEventsConverter(
-            [],
-            self.fake_plugin_mgr,
-            add_catchall=True)
+            self.CONF, [], self.fake_plugin_mgr)
         self.assertEqual(1, len(c.definitions))
-        e = c.to_event(self.test_notification1)
+        e = c.to_event('INFO', self.test_notification1)
         self.assertIsValidEvent(e, self.test_notification1)
         self.assertEqual(1, len(e.traits))
         self.assertHasDefaultTraits(e)
 
-        e = c.to_event(self.test_notification2)
+        e = c.to_event('INFO', self.test_notification2)
         self.assertIsValidEvent(e, self.test_notification2)
         self.assertEqual(1, len(e.traits))
         self.assertHasDefaultTraits(e)
 
     def test_converter_empty_cfg_without_catchall(self):
+        self.CONF.set_override('drop_unmatched_notifications', True,
+                               group='event')
         c = converter.NotificationEventsConverter(
-            [],
-            self.fake_plugin_mgr,
-            add_catchall=False)
+            self.CONF, [], self.fake_plugin_mgr)
         self.assertEqual(0, len(c.definitions))
-        e = c.to_event(self.test_notification1)
+        e = c.to_event('INFO', self.test_notification1)
         self.assertIsNotValidEvent(e, self.test_notification1)
 
-        e = c.to_event(self.test_notification2)
+        e = c.to_event('INFO', self.test_notification2)
         self.assertIsNotValidEvent(e, self.test_notification2)
 
     @staticmethod
     def _convert_message(convert, level):
-        message = {'priority': level, 'event_type': "foo",
-                   'message_id': "abc", 'publisher_id': "1"}
-        return convert.to_event(message)
+        message = {'priority': level, 'event_type': "foo", 'publisher_id': "1",
+                   'metadata': {'message_id': "abc",
+                                'timestamp': "2013-08-08 21:06:37.803826"}}
+        return convert.to_event(level, message)
 
     def test_store_raw_all(self):
-        self.CONF.event.store_raw = ['info', 'error']
+        self.CONF.set_override('store_raw', ['info', 'error'],
+                               group='event')
         c = converter.NotificationEventsConverter(
-            [], self.fake_plugin_mgr)
+            self.CONF, [], self.fake_plugin_mgr)
         self.assertTrue(self._convert_message(c, 'info').raw)
         self.assertTrue(self._convert_message(c, 'error').raw)
 
     def test_store_raw_info_only(self):
-        self.CONF.event.store_raw = ['info']
+        self.CONF.set_override('store_raw', ['info'],
+                               group='event')
         c = converter.NotificationEventsConverter(
-            [], self.fake_plugin_mgr)
+            self.CONF, [], self.fake_plugin_mgr)
         self.assertTrue(self._convert_message(c, 'info').raw)
         self.assertFalse(self._convert_message(c, 'error').raw)
 
     def test_store_raw_error_only(self):
-        self.CONF.event.store_raw = ['error']
+        self.CONF.set_override('store_raw', ['error'],
+                               group='event')
         c = converter.NotificationEventsConverter(
-            [], self.fake_plugin_mgr)
+            self.CONF, [], self.fake_plugin_mgr)
         self.assertFalse(self._convert_message(c, 'info').raw)
         self.assertTrue(self._convert_message(c, 'error').raw)
 
     def test_store_raw_skip_all(self):
         c = converter.NotificationEventsConverter(
-            [], self.fake_plugin_mgr)
+            self.CONF, [], self.fake_plugin_mgr)
         self.assertFalse(self._convert_message(c, 'info').raw)
         self.assertFalse(self._convert_message(c, 'error').raw)
 
     def test_store_raw_info_only_no_case(self):
-        self.CONF.event.store_raw = ['INFO']
+        self.CONF.set_override('store_raw', ['INFO'],
+                               group='event')
         c = converter.NotificationEventsConverter(
-            [], self.fake_plugin_mgr)
+            self.CONF, [], self.fake_plugin_mgr)
         self.assertTrue(self._convert_message(c, 'info').raw)
         self.assertFalse(self._convert_message(c, 'error').raw)
 
     def test_store_raw_bad_skip_all(self):
-        self.CONF.event.store_raw = ['unknown']
+        self.CONF.set_override('store_raw', ['unknown'],
+                               group='event')
         c = converter.NotificationEventsConverter(
-            [], self.fake_plugin_mgr)
+            self.CONF, [], self.fake_plugin_mgr)
         self.assertFalse(self._convert_message(c, 'info').raw)
         self.assertFalse(self._convert_message(c, 'error').raw)
 
     def test_store_raw_bad_and_good(self):
-        self.CONF.event.store_raw = ['info', 'unknown']
+        self.CONF.set_override('store_raw', ['info', 'unknown'],
+                               group='event')
         c = converter.NotificationEventsConverter(
-            [], self.fake_plugin_mgr)
+            self.CONF, [], self.fake_plugin_mgr)
         self.assertTrue(self._convert_message(c, 'info').raw)
         self.assertFalse(self._convert_message(c, 'error').raw)
 
-    def test_setup_events_default_config(self):
+    @mock.patch('ceilometer.declarative.LOG')
+    def test_setup_events_load_config_in_code_tree(self, mocked_log):
         self.CONF.set_override('definitions_cfg_file',
                                '/not/existing/file', group='event')
         self.CONF.set_override('drop_unmatched_notifications',
                                False, group='event')
 
-        c = converter.setup_events(self.fake_plugin_mgr)
+        c = converter.setup_events(self.CONF, self.fake_plugin_mgr)
         self.assertIsInstance(c, converter.NotificationEventsConverter)
-        self.assertEqual(1, len(c.definitions))
-        self.assertTrue(c.definitions[0].is_catchall)
-
-        self.CONF.set_override('drop_unmatched_notifications',
-                               True, group='event')
-
-        c = converter.setup_events(self.fake_plugin_mgr)
-        self.assertIsInstance(c, converter.NotificationEventsConverter)
-        self.assertEqual(0, len(c.definitions))
+        log_called_args = mocked_log.debug.call_args_list
+        self.assertEqual(
+            'No Definitions configuration file found! Using default config.',
+            log_called_args[0][0][0])
+        self.assertTrue(log_called_args[1][0][0].startswith(
+            'Loading definitions configuration file:'))
