@@ -14,12 +14,12 @@
 # under the License.
 
 import os
+from unittest import mock
 import uuid
 
 import fixtures
 from gnocchiclient import exceptions as gnocchi_exc
 from keystoneauth1 import exceptions as ka_exceptions
-import mock
 from oslo_config import fixture as config_fixture
 from oslo_utils import fileutils
 from oslo_utils import fixture as utils_fixture
@@ -44,12 +44,13 @@ INSTANCE_DELETE_START = models.Event(
             models.Trait(
                 'user_id', 1, u'1e3ce043029547f1a61c1996d1a531a2'),
             models.Trait('service', 1, u'compute'),
+            models.Trait('availability_zone', 1, u'zone1'),
             models.Trait('disk_gb', 2, 0),
             models.Trait('instance_type', 1, u'm1.tiny'),
             models.Trait('tenant_id', 1, u'7c150a59fe714e6f9263774af9688f0e'),
             models.Trait('root_gb', 2, 0),
             models.Trait('ephemeral_gb', 2, 0),
-            models.Trait('instance_type_id', 2, 2),
+            models.Trait('instance_type_id', 2, u'2'),
             models.Trait('vcpus', 2, 1),
             models.Trait('memory_mb', 2, 512),
             models.Trait(
@@ -64,6 +65,33 @@ INSTANCE_DELETE_START = models.Event(
     message_id=u'a15b94ee-cb8e-4c71-9abe-14aa80055fb4',
 )
 
+INSTANCE_CREATE_END = models.Event(
+    event_type=u'compute.instance.create.end',
+    traits=[models.Trait('state', 1, u'active'),
+            models.Trait(
+                'user_id', 1, u'1e3ce043029547f1a61c1996d1a531a2'),
+            models.Trait('service', 1, u'compute'),
+            models.Trait('availability_zone', 1, u'zone1'),
+            models.Trait('disk_gb', 2, 0),
+            models.Trait('instance_type', 1, u'm1.tiny'),
+            models.Trait('tenant_id', 1, u'7c150a59fe714e6f9263774af9688f0e'),
+            models.Trait('root_gb', 2, 0),
+            models.Trait('ephemeral_gb', 2, 0),
+            models.Trait('instance_type_id', 2, u'2'),
+            models.Trait('vcpus', 2, 1),
+            models.Trait('memory_mb', 2, 512),
+            models.Trait(
+                'instance_id', 1, u'9f9d01b9-4a58-4271-9e27-398b21ab20d1'),
+            models.Trait('host', 1, u'vagrant-precise'),
+            models.Trait(
+                'request_id', 1, u'req-fb3c4546-a2e5-49b7-9fd2-a63bd658bc39'),
+            models.Trait('project_id', 1, u'7c150a59fe714e6f9263774af9688f0e'),
+            models.Trait('launched_at', 4, '2012-05-08T20:23:47')],
+    raw={},
+    generated='2012-05-08T20:24:14.824743',
+    message_id=u'202f745e-4913-11e9-affe-9797342bd3a8',
+)
+
 IMAGE_DELETE_START = models.Event(
     event_type=u'image.delete',
     traits=[models.Trait(u'status', 1, u'deleted'),
@@ -83,8 +111,8 @@ IMAGE_DELETE_START = models.Event(
 )
 
 
-VOLUME_DELETE_START = models.Event(
-    event_type=u'volume.delete.start',
+VOLUME_DELETE_END = models.Event(
+    event_type=u'volume.delete.end',
     traits=[models.Trait(u'availability_zone', 1, u'nova'),
             models.Trait(u'created_at', 1, u'2016-11-28T13:19:53+00:00'),
             models.Trait(u'display_name', 1, u'vol-001'),
@@ -125,6 +153,28 @@ FLOATINGIP_DELETE_END = models.Event(
     message_id=u'a15b94ee-cb8e-4c71-9abe-14aa80055fb4'
 )
 
+VOLUME_TRANSFER_ACCEPT_END = models.Event(
+    event_type='volume.transfer.accept.end',
+    traits=[models.Trait(u'tenant_id', 1, '945e7d09220e4308abe4b3b734bf5fce>'),
+            models.Trait(u'project_id', 1, '85bc015f7a2342348593077a927c4aaa'),
+            models.Trait(u'user_id', 1, '945e7d09220e4308abe4b3b734bf5fce'),
+            models.Trait(u'service', 1, 'volume.controller-0'),
+            models.Trait(
+                u'request_id', 1, 'req-71dd1ae4-81ca-431a-b9fd-ac833eba889f'),
+            models.Trait(
+                u'resource_id', 1, '156b8d3f-ad99-429b-b84c-3f263fb2a801'),
+            models.Trait(
+                u'display_name', 1, 'test-vol'),
+            models.Trait(
+                u'type', 1, 'req-71dd1ae4-81ca-431a-b9fd-ac833eba889f'),
+            models.Trait(u'host', 1, 'hostgroup@tripleo_iscsi#tripleo_iscsi'),
+            models.Trait(u'created_at', 4, '2020-08-28 12:51:52'),
+            models.Trait(u'size', 2, 1)],
+    raw={},
+    generated='2020-08-28T12:52:22.930413',
+    message_id=u'9fc4ceee-d980-4098-a685-2ad660838ac1'
+)
+
 
 class PublisherTest(base.BaseTestCase):
 
@@ -178,6 +228,9 @@ class PublisherTest(base.BaseTestCase):
         self.useFixture(fixtures.MockPatch(
             'gnocchiclient.v1.client.Client',
             return_value=mock.Mock()))
+        self.useFixture(fixtures.MockPatch(
+            'ceilometer.keystone_client.get_session',
+            return_value=mock.Mock()))
         self.ks_client = ks_client
 
     def test_config_load(self):
@@ -462,10 +515,13 @@ class PublisherWorkflowTest(base.BaseTestCase,
         self.useFixture(fixtures.MockPatch(
             'ceilometer.keystone_client.get_client',
             return_value=ks_client))
+        self.useFixture(fixtures.MockPatch(
+            'ceilometer.keystone_client.get_session',
+            return_value=ks_client))
         self.ks_client = ks_client
 
     @mock.patch('gnocchiclient.v1.client.Client')
-    def test_event_workflow(self, fakeclient_cls):
+    def test_delete_event_workflow(self, fakeclient_cls):
         url = netutils.urlsplit("gnocchi://")
         self.publisher = gnocchi.GnocchiPublisher(self.conf.conf, url)
 
@@ -515,12 +571,61 @@ class PublisherWorkflowTest(base.BaseTestCase,
 
         self.publisher.publish_events([INSTANCE_DELETE_START,
                                        IMAGE_DELETE_START,
-                                       VOLUME_DELETE_START,
+                                       VOLUME_DELETE_END,
                                        FLOATINGIP_DELETE_END])
         self.assertEqual(8, len(fakeclient.mock_calls))
         for call in expected_calls:
             self.assertIn(call, fakeclient.mock_calls)
 
+    @mock.patch('gnocchiclient.v1.client.Client')
+    def test_create_event_workflow(self, fakeclient_cls):
+        url = netutils.urlsplit("gnocchi://")
+        self.publisher = gnocchi.GnocchiPublisher(self.conf.conf, url)
+
+        fakeclient = fakeclient_cls.return_value
+
+        now = timeutils.utcnow()
+        self.useFixture(utils_fixture.TimeFixture(now))
+
+        expected_calls = [
+            mock.call.resource.create(
+                'instance',
+                {'id': '9f9d01b9-4a58-4271-9e27-398b21ab20d1',
+                 'user_id': '1e3ce043029547f1a61c1996d1a531a2',
+                 'project_id': '7c150a59fe714e6f9263774af9688f0e',
+                 'availability_zone': 'zone1',
+                 'flavor_name': 'm1.tiny',
+                 'flavor_id': '2',
+                 'host': 'vagrant-precise'}),
+        ]
+
+        self.publisher.publish_events([INSTANCE_CREATE_END])
+        self.assertEqual(1, len(fakeclient.mock_calls))
+        for call in expected_calls:
+            self.assertIn(call, fakeclient.mock_calls)
+
+    @mock.patch('gnocchiclient.v1.client.Client')
+    def test_update_event_workflow(self, fakeclient_cls):
+        url = netutils.urlsplit("gnocchi://")
+        self.publisher = gnocchi.GnocchiPublisher(self.conf.conf, url)
+
+        fakeclient = fakeclient_cls.return_value
+
+        now = timeutils.utcnow()
+        self.useFixture(utils_fixture.TimeFixture(now))
+
+        expected_calls = [
+            mock.call.resource.update(
+                'volume',
+                '156b8d3f-ad99-429b-b84c-3f263fb2a801',
+                {'project_id': '85bc015f7a2342348593077a927c4aaa'}),
+        ]
+
+        self.publisher.publish_events([VOLUME_TRANSFER_ACCEPT_END])
+        self.assertEqual(1, len(fakeclient.mock_calls))
+        for call in expected_calls:
+            self.assertIn(call, fakeclient.mock_calls)
+
     @mock.patch('ceilometer.publisher.gnocchi.LOG')
     @mock.patch('gnocchiclient.v1.client.Client')
     def test_workflow(self, fakeclient_cls, logger):
@@ -532,8 +637,24 @@ class PublisherWorkflowTest(base.BaseTestCase,
         gnocchi_id = uuid.uuid4()
 
         expected_calls = [
-            mock.call.archive_policy.get("ceilometer-low"),
-            mock.call.archive_policy.get("ceilometer-low-rate"),
+            mock.call.archive_policy.create({"name": "ceilometer-low",
+                                             "back_window": 0,
+                                             "aggregation_methods": ["mean"],
+                                             "definition": mock.ANY}),
+            mock.call.archive_policy.create({"name": "ceilometer-low-rate",
+                                             "back_window": 0,
+                                             "aggregation_methods": [
+                                                 "mean", "rate:mean"],
+                                             "definition": mock.ANY}),
+            mock.call.archive_policy.create({"name": "ceilometer-high",
+                                             "back_window": 0,
+                                             "aggregation_methods": ["mean"],
+                                             "definition": mock.ANY}),
+            mock.call.archive_policy.create({"name": "ceilometer-high-rate",
+                                             "back_window": 0,
+                                             "aggregation_methods": [
+                                                 "mean", "rate:mean"],
+                                             "definition": mock.ANY}),
             mock.call.metric.batch_resources_metrics_measures(
                 {resource_id: {metric_name: self.metric_attributes}},
                 create_metrics=True)
@@ -541,6 +662,8 @@ class PublisherWorkflowTest(base.BaseTestCase,
         expected_debug = [
             mock.call('filtered project found: %s',
                       'a2d42c23-d518-46b6-96ab-3fba2e146859'),
+            mock.call('Processing sample [%s] for resource ID [%s].',
+                      self.sample, resource_id),
         ]
 
         measures_posted = False
